from django.utils.encoding import smart_unicode, smart_str, force_unicode
from django import forms

from survey import widgets
from survey import fields

class QuestionTypeBase(type):

    def __new__(cls, name, bases, attrs):
        return super(QuestionTypeBase, cls).__new__(cls, name, bases, attrs)


class QuestionType(object):
    id = None

    __metaclass__ = QuestionTypeBase

    def __init__(self, instance=None):
        if instance:
            self.load(instance)
        self.instance = instance

    def load(self, instance):
        pass

    def clean_params(self, params):
        return params

    def clean_data(self, data):
        raise NotImplemented

    def validate_answer(self, answer):
        raise NotImplemented

    def store_answer(self, answer):
        raise NotImplemented

    def formfield(self):
        kwargs = {
            'label': self.instance.title,
            'required': self.instance.required,
            'help_text': self.instance.help_text,
            'widget': forms.TextInput
        }
        field = forms.CharField(**kwargs)
        return field

    def answer_display(self, answer):
        return force_unicode(answer)


class Range(QuestionType):

    def clean_data(self, data):
        params = {}
        if not ['max', 'min'] == data.keys():
            raise Exception("Invalid range data")

        for key in ['max', 'min']:
            value = data[key]['value']
            for lang, text in data[key]['text'].iteritems():
                if not lang in params:
                    params[lang] = {}
                params[lang][key] = {'value': value, 'text': text}
        return params

    def formfield(self):
        field_attrs = {}
        min = self.instance.data['min']
        max = self.instance.data['max']
        keys = range(min['value'], max['value']+1)
        labels = map(str, keys)
        choices = zip(keys, labels)
        kwargs = {
            'label': self.instance.title,
            'required': self.instance.required,
            'help_text': self.instance.help_text,
            'widget': widgets.RangeSelect,
            'choices': choices
        }
        field = forms.ChoiceField(**kwargs)
        field.widget.attrs.update({
           'min_text': min['text'],
           'max_text': max['text']
        })
        return field


class Text(QuestionType):

    def clean_data(self, data):
        return {}

    def formfield(self):
        kwargs = {
            'label': self.instance.title,
            'required': self.instance.required,
            'help_text': self.instance.help_text,
            'widget': forms.Textarea
        }
        field = forms.CharField(**kwargs)
        return field


class Choice(QuestionType):

    answer_delimiter = u"|"

    def _parse_answer(self, data):
        def _inner(a):
            if not a:
                return u''
            if type(a) in [list, tuple]:
                return "%s::%s" % (data[a[0]], a[1])
            return data[a]
        return _inner

    def answer_display(self, answers):
        data = dict(map(lambda c:(c['id'], c['text']), self.instance.data))
        if self.instance.params.get('allow_multiple'):
            return self.answer_delimiter.join(map(self._parse_answer(data),
                                                  answers))

        answer = self._parse_answer(data)(answers)
        return force_unicode(answer)

    def formfield(self):
        choices = []
        input_choices = []
        for choice in self.instance.data:
            choices.append((choice['id'], choice['text']))
            if choice.get('custom'):
                input_choices.append(choice['id'])

        select_widget = self.instance.params.get('widget') == 'select'
        widget_attrs = {}
        field_attrs = {}

        if select_widget:
            self.instance.required = True
            choices = [('', '-------')] + choices
            widget = widgets.Select
        else:
            widget = widgets.RadioSelect

        if self.instance.params.get('allow_multiple'):
            if select_widget:
                widget = widgets.SelectMultiple
            else:
                widget = widgets.CheckboxSelectMultiple

        if self.instance.required:
            widget_attrs['class'] = 'required'

        kwargs = {
            'label': self.instance.title,
            'required': self.instance.required,
            'help_text': self.instance.help_text,
            'widget': widget,
            'choices': choices
        }

        field_cls = forms.ChoiceField
        if self.instance.params.get('allow_multiple'):
            field_cls = forms.MultipleChoiceField

        if len(input_choices):
            kwargs['input_choices'] = input_choices
            field_cls = fields.ChoiceWithInputField
            kwargs['widget'] = widgets.RadioSelectWithInput
            if self.instance.params.get('allow_multiple'):
                field_cls = fields.MultipleChoiceWithInputField
                kwargs['widget'] = widgets.CheckboxSelectMultipleWithInput

        field = field_cls(**kwargs)
        field.widget.attrs.update(widget_attrs)
        return field

    def clean_data(self, data):
        answers = {}
        for answer in data['choices']:
            id = answer['id']
            for lang, text in answer.get('text').iteritems():
                if not lang in answers:
                    answers[lang] = []

                params = {
                    'id':id,
                    'text': text,
                    'custom': answer.get('custom', False)
                }
                answers[lang].append(params)
        return answers


QUESTION_TYPES = {
    'range': Range,
    'choice': Choice,
    'text': Text
}

QUESTION_TYPE_CHOICES = (('choice', 'Choice'),
                         ('text', 'Text'),
                         ('range', 'Range'))
