from itertools import chain

from django.utils.encoding import smart_unicode, smart_str, force_unicode
from django.utils.datastructures import MultiValueDict, MergeDict
from django.utils.safestring import mark_safe
from django.utils.encoding import force_unicode
from django.utils.html import escape, conditional_escape
from django.forms import widgets
from django import forms
from django.forms.widgets import Select, RadioSelect, CheckboxSelectMultiple, \
                                 CheckboxInput


class RadioInputWithText(widgets.RadioInput):

    def __init__(self, *args, **kwargs):
        self.input_index = kwargs.pop("input_index", 0)
        super(RadioInputWithText, self).__init__(*args, **kwargs)

    def is_checked(self):
        if not self.value:
            return False
        return self.value[0] == self.choice_value

    def render(self, name=None, value=None, attrs=None, choices=()):
        name = name or self.name
        value = value or self.value
        attrs = attrs or self.attrs
        if 'id' in self.attrs:
            label_for = ' for="%s_%s"' % (self.attrs['id'], self.index)
        else:
            label_for = ''
        input_value = ''
        if value:
            input_value = value[1]
        choice_label = conditional_escape(force_unicode(self.choice_label))
        extra = u'<input name="%s" value="%s" type="text"/>' % ("%s_%d_input" % (name, self.input_index), input_value)
        return mark_safe(u'<label%s>%s %s%s</label>' % (label_for, self.tag(), choice_label, extra))


class RadioSelectWithInputRenderer(widgets.RadioFieldRenderer):

    def __iter__(self):
        for i, choice in enumerate(self.choices):
            if choice[0] in self.input_choices:
                input_index = self.input_choices.index(choice[0])
                widget = RadioInputWithText(self.name, self.value,
                                            self.attrs.copy(), choice, i,
                                            input_index=input_index)
            else:
                widget = widgets.RadioInput(self.name, self.value,
                                            self.attrs.copy(), choice, i)
            yield widget


class RadioSelectWithInput(RadioSelect):
    renderer = RadioSelectWithInputRenderer

    def get_renderer(self, name, value, attrs=None, choices=()):
        """Returns an instance of the renderer."""
        if value is None: value = ''
        if type(value) in [list, tuple]:
            value = list(value)
            if value[0] is None: value[0] = u''
            if value[1] is None: value[1] = u''
            funi = force_unicode
            str_value = (funi(value[0]), funi(value[1]))
        else:
            str_value = force_unicode(value) # Normalize to string.
        final_attrs = self.build_attrs(attrs)
        choices = list(chain(self.choices, choices))
        renderer = self.renderer(name, str_value, final_attrs, choices)
        renderer.input_choices = self.input_choices
        return renderer

    def value_from_datadict(self, data, files, name):
        value = super(RadioSelectWithInput,
                      self).value_from_datadict(data, files, name)
        if value in self.input_choices:
            index = self.input_choices.index(value)
            input_name = "%s_%d_input" % (name, index)
            input_value = data.get(input_name, '')
            value = (value, input_value)
        return value


class CheckboxSelectMultipleWithInput(CheckboxSelectMultiple):

    def value_from_datadict(self, data, files, name):
        values = []
        if isinstance(data, (MultiValueDict, MergeDict)):
            values = data.getlist(name)
        else:
            values = data.get(name, None)
        if values:
            for i, value in enumerate(values):
                if value in self.input_choices:
                    index = self.input_choices.index(value)
                    input_name = "%s_%d_input" % (name, index)
                    input_value = data.get(input_name, '')
                    values[i] = (value, input_value)
        return values

    def render(self, name, value, attrs=None, choices=()):
        if value is None: value = []
        has_id = attrs and 'id' in attrs
        final_attrs = self.build_attrs(attrs, name=name)
        output = [u'<ul>']
        # Normalize to strings
        str_values = set([force_unicode(v[0]) if type(v) in [list, tuple] else force_unicode(v) for v in value])
        str_inputs = dict(v for v in value if type(v) in [list, tuple])
        for i, (option_value, option_label) in enumerate(chain(self.choices, choices)):
            # If an ID attribute was given, add a numeric index as a suffix,
            # so that the checkboxes don't all have the same ID attribute.
            if has_id:
                final_attrs = dict(final_attrs, id='%s_%s' % (attrs['id'], i))
                label_for = u' for="%s"' % final_attrs['id']
            else:
                label_for = ''

            extra = ""
            if option_value in self.input_choices:
                input_value = str_inputs.get(option_value, '')
                index = self.input_choices.index(option_value)
                extra = u'<input name="%s" value="%s" type="text"/>' % ("%s_%d_input" % (name, index), input_value)
            cb = CheckboxInput(final_attrs, check_test=lambda value: value in str_values)
            option_value = force_unicode(option_value)
            rendered_cb = cb.render(name, option_value)
            option_label = conditional_escape(force_unicode(option_label))
            output.append(u'<li><label%s>%s %s%s</label></li>' % (label_for, rendered_cb, option_label, extra))
        output.append(u'</ul>')
        return mark_safe(u'\n'.join(output))


class RangeSelectRenderer(widgets.RadioFieldRenderer):

    def render(self):
        max_text = self.attrs.get('max_text')
        del self.attrs['max_text']
        min_text = self.attrs.get('min_text')
        del self.attrs['min_text']

        html = u"<table class='range'><tbody>" \
                "<tr>%s</tr><tr>%s</tr></tbody></table>"
        top = u"\n".join([u"<td><label for='%s_%s'>%s</label></td>" % \
            (self.attrs.get('id'),
             w[0]-1, force_unicode(w[1])) for w in self.choices])
        top = u"<td></td>" + top + u"<td></td>"

        bottom_content = ""
        for w in self:
            w.choice_label = ""
            bottom_content += "<td>%s</td>" % force_unicode(w)

        bottom = "<td>%s</td>%s<td>%s</td>" % \
            (min_text,
             bottom_content,
             max_text)

        return mark_safe(html % (top, bottom))


class RangeSelect(RadioSelect):
    renderer = RangeSelectRenderer
